@@ -13,7 +13,7 @@ implementation (like GTK, Qt, or CLI).
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import glob, sys, os.path, optparse, traceback, locale, gettext, re, io
+import glob, sys, os.path, optparse, traceback, locale, gettext, re
 import errno, zlib
 import subprocess, threading, webbrowser
 import signal
@@ -33,7 +33,7 @@ else:
     from configparser import ConfigParser
     PY3 = True
 
-__version__ = '2.20.9'
+__version__ = '2.20.10'
 
 
 def excstr(exception):
@@ -112,7 +112,7 @@ def thread_collect_info(report, reportfile, package, ui, symptom_script=None,
             report['Symptom'] = os.path.splitext(os.path.basename(symptom_script))[0]
         except StopIteration:
             sys.exit(0)
-        except Exception as e:
+        except Exception:
             apport.error('symptom script %s crashed:', symptom_script)
             traceback.print_exc()
             sys.exit(0)
@@ -243,11 +243,8 @@ class UserInterface:
             logind_session = None
         else:
             reports = apport.fileutils.get_new_reports()
-            if PY3:
-                proc_pid_fd = os.open('/proc/%s' % os.getpid(), os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
-                logind_session = apport.Report.get_logind_session(proc_pid_fd=proc_pid_fd)
-            else:
-                logind_session = apport.Report.get_logind_session(os.getpid())
+            proc_pid_fd = os.open('/proc/%s' % os.getpid(), os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
+            logind_session = apport.Report.get_logind_session(proc_pid_fd)
 
         for f in reports:
             if not self.load_report(f):
@@ -259,6 +256,9 @@ class UserInterface:
             # and the session started before the report's "Date".
             if logind_session and '_LogindSession' in self.report and \
                'Date' in self.report:
+                # report.get_timestamp() can return None
+                if not self.report.get_timestamp():
+                    continue
                 if logind_session[0] != self.report['_LogindSession'] or \
                    logind_session[1] > self.report.get_timestamp():
                     continue
@@ -395,7 +395,17 @@ class UserInterface:
         mark the report for uploading.
         '''
         self.report = apport.Report('Hang')
-        self.report.add_proc_info(pid)
+        try:
+            self.report.add_proc_info(pid)
+        except ValueError as e:
+            if str(e) == 'invalid process':
+                self.ui_error_message(_('Invalid PID'),
+                                      _('The specified process ID does not exist.'))
+                sys.exit(1)
+            elif str(e) == 'not accessible':
+                self.ui_error_message(_('Not your PID'),
+                                      _('The specified process ID does not belong to you.'))
+                sys.exit(1)
         self.report.add_package_info()
         path = self.report.get('ExecutablePath', '')
         self.cur_package = apport.fileutils.find_file_package(path)
@@ -454,20 +464,15 @@ class UserInterface:
         # if PID is given, add info
         if self.options.pid:
             try:
-                proc_pid_fd = None
-                if PY3:
-                    proc_pid_fd = os.open('/proc/%s' % self.options.pid, os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
-                    stat_file = os.open('stat', os.O_RDONLY | os.O_CLOEXEC, dir_fd=proc_pid_fd)
-                else:
-                    stat_file = '/proc/%s/stat' % self.options.pid
-                with io.open(stat_file) as f:
+                proc_pid_fd = os.open('/proc/%s' % self.options.pid, os.O_RDONLY | os.O_PATH | os.O_DIRECTORY)
+                with open('stat', opener=lambda path, mode: os.open(path, mode, dir_fd=proc_pid_fd)) as f:
                     stat = f.read().split()
                 flags = int(stat[8])
                 if flags & PF_KTHREAD:
                     # this PID is a kernel thread
                     self.options.package = 'linux'
                 else:
-                    self.report.add_proc_info(pid=self.options.pid, proc_pid_fd=proc_pid_fd)
+                    self.report.add_proc_info(proc_pid_fd=proc_pid_fd)
             except (ValueError, IOError, OSError) as e:
                 if hasattr(e, 'errno'):
                     # silently ignore nonexisting PIDs; the user must not close the
@@ -528,7 +533,7 @@ class UserInterface:
                 self.ui_error_message(_('Cannot create report'), excstr(e))
         else:
             # show what's being sent
-            allowed_to_report = True
+            allowed_to_report = apport.fileutils.allowed_to_report()
             response = self.ui_present_report_details(allowed_to_report)
             if response['report']:
                 self.file_report()
@@ -607,7 +612,7 @@ class UserInterface:
             return False
 
         # show what's being sent
-        allowed_to_report = True
+        allowed_to_report = apport.fileutils.allowed_to_report()
         response = self.ui_present_report_details(allowed_to_report)
         if response['report']:
             self.crashdb.update(self.options.update_report, self.report,
@@ -1093,7 +1098,7 @@ class UserInterface:
                                                     args=(self.report, self.report_file, self.cur_package,
                                                           hookui, symptom_script, ignore_uninstalled))
                 icthread.start()
-                while icthread.isAlive():
+                while icthread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     if hookui:
                         try:
@@ -1158,7 +1163,7 @@ class UserInterface:
                 bpthread = apport.REThread.REThread(target=self.report.search_bug_patterns,
                                                     args=(self.crashdb.get_bugpattern_baseurl(),))
                 bpthread.start()
-                while bpthread.isAlive():
+                while bpthread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     try:
                         bpthread.join(0.1)
@@ -1179,7 +1184,7 @@ class UserInterface:
                 known_thread = apport.REThread.REThread(target=self.crashdb.known,
                                                         args=(self.report,))
                 known_thread.start()
-                while known_thread.isAlive():
+                while known_thread.is_alive():
                     self.ui_pulse_info_collection_progress()
                     try:
                         known_thread.join(0.1)
@@ -1197,7 +1202,7 @@ class UserInterface:
             # might damage the stack trace
             anonymize_thread = apport.REThread.REThread(target=self.report.anonymize)
             anonymize_thread.start()
-            while anonymize_thread.isAlive():
+            while anonymize_thread.is_alive():
                 self.ui_pulse_info_collection_progress()
                 try:
                     anonymize_thread.join(0.1)
@@ -1276,7 +1281,7 @@ class UserInterface:
         try:
             try:
                 subprocess.call(sudo_prefix + ['xdg-open', url])
-            except OSError as e:
+            except OSError:
                 # fall back to webbrowser
                 webbrowser.open(url, new=True, autoraise=True)
                 sys.exit(0)
@@ -1324,7 +1329,7 @@ class UserInterface:
         upthread = apport.REThread.REThread(target=self.crashdb.upload,
                                             args=(self.report, progress_callback))
         upthread.start()
-        while upthread.isAlive():
+        while upthread.is_alive():
             self.ui_set_upload_progress(__upload_progress)
             try:
                 upthread.join(0.1)
