@@ -28,15 +28,17 @@ import weakref
 
 try:
     from typing import (Any, Callable, Dict, Iterator, List, Optional,
-                        Set, Tuple, cast)
+                        Set, Tuple, Union, cast, KeysView)
     Any  # pyflakes
     Callable  # pyflakes
     Dict  # pyflakes
     Iterator  # pyflakes
+    KeysView  # pyflakes
     List  # pyflakes
     Optional  # pyflakes
     Set  # pyflakes
     Tuple  # pyflakes
+    Union  # pyflakes
 except ImportError:
     def cast(typ, obj):  # type: ignore
         return obj
@@ -68,6 +70,29 @@ class CacheClosedException(Exception):
     """Exception that is thrown when the cache is used after close()."""
 
 
+class _WrappedLock(object):
+    """Wraps an apt_pkg.FileLock to raise LockFailedException.
+
+    Initialized using a directory path."""
+
+    def __init__(self, path):
+        # type: (str) -> None
+        self._path = path
+        self._lock = apt_pkg.FileLock(os.path.join(path, "lock"))
+
+    def __enter__(self):
+        # type: () -> None
+        try:
+            return self._lock.__enter__()
+        except apt_pkg.Error as e:
+            raise LockFailedException(("Failed to lock directory %s: %s") %
+                                       (self._path, e))
+
+    def __exit__(self, typ, value, traceback):
+        # type: (object, object, object) -> None
+        return self._lock.__exit__(typ, value, traceback)
+
+
 class Cache(object):
     """Dictionary-like package cache.
 
@@ -100,8 +125,8 @@ class Cache(object):
         self._depcache = cast(apt_pkg.DepCache, None)  # type: apt_pkg.DepCache
         self._records = cast(apt_pkg.PackageRecords, None)  # type: apt_pkg.PackageRecords # nopep8
         self._list = cast(apt_pkg.SourceList, None)  # type: apt_pkg.SourceList
-        self._callbacks = {}  # type: Dict[str, List[Callable[..., None]]]
-        self._callbacks2 = {}  # type: Dict[str, List[Tuple[Callable[..., None], List[Any], Dict[Any,Any]]]] # nopep8
+        self._callbacks = {}  # type: Dict[str, List[Union[Callable[..., None],str]]] # nopep8
+        self._callbacks2 = {}  # type: Dict[str, List[Tuple[Callable[..., Any], Tuple[Any, ...], Dict[Any,Any]]]] # nopep8
         self._weakref = weakref.WeakValueDictionary()  # type: weakref.WeakValueDictionary[str, apt.Package] # nopep8
         self._weakversions = weakref.WeakSet()  # type: weakref.WeakSet[Version] # nopep8
         self._changes_count = -1
@@ -133,8 +158,18 @@ class Cache(object):
             # Call InitSystem so the change to Dir::State::Status is actually
             # recognized (LP: #320665)
             apt_pkg.init_system()
+
+        # Prepare a lock object (context manager for archive lock)
+        archive_dir = apt_pkg.config.find_dir("Dir::Cache::Archives")
+        self._archive_lock = _WrappedLock(archive_dir)
+
         self.open(progress)
 
+    def fix_broken(self):
+        # type: () -> None
+        """Fix broken packages."""
+        self._depcache.fix_broken()
+
     def _inc_changes_count(self):
         # type: () -> None
         """Increase the number of changes"""
@@ -170,7 +205,7 @@ class Cache(object):
                 if callback == '_inc_changes_count':
                     self._inc_changes_count()
                 else:
-                    callback()
+                    callback()  # type: ignore
 
         if name in self._callbacks2:
             for callback, args, kwds in self._callbacks2[name]:
@@ -246,29 +281,26 @@ class Cache(object):
         return self
 
     def __exit__(self, exc_type, exc_value, traceback):
+        # type: (object, object, object) -> None
         """ Exit the with statement """
         self.close()
 
     def __getitem__(self, key):
         # type: (object) -> Package
         """ look like a dictionary (get key) """
-        key = str(key)
         try:
-            return self._weakref[key]
+            key = str(key)
+            rawpkg = self._cache[key]
         except KeyError:
-            try:
-                rawpkg = self._cache[key]
-            except KeyError:
-                raise KeyError('The cache has no package named %r' % key)
+            raise KeyError('The cache has no package named %r' % key)
 
-            # It might be excluded due to not having a version or something
-            if not self.__is_real_pkg(rawpkg):
-                raise KeyError('The cache has no package named %r' % key)
+        # It might be excluded due to not having a version or something
+        if not self.__is_real_pkg(rawpkg):
+            raise KeyError('The cache has no package named %r' % key)
 
-            pkg = self._rawpkg_to_pkg(rawpkg)
-            self._weakref[key] = pkg
+        pkg = self._rawpkg_to_pkg(rawpkg)
 
-            return pkg
+        return pkg
 
     def get(self, key, default=None):
         # type: (object, object) -> Any
@@ -282,17 +314,14 @@ class Cache(object):
             return default
 
     def _rawpkg_to_pkg(self, rawpkg):
+        # type: (apt_pkg.Package) -> Package
         """Returns the apt.Package object for an apt_pkg.Package object.
 
         .. versionadded:: 1.0.0
         """
-        fullname = rawpkg.get_fullname(pretty=False)
-        try:
-            pkg = self._weakref[fullname]
-        except KeyError:
-            pkg = Package(self, rawpkg)
-            self._weakref[fullname] = pkg
-        return pkg
+        fullname = rawpkg.get_fullname(pretty=True)
+
+        return self._weakref.setdefault(fullname, Package(self, rawpkg))
 
     def __iter__(self):
         # type: () -> Iterator[Package]
@@ -302,9 +331,11 @@ class Cache(object):
         # is disastrous if we use compressed package indexes, and slower than
         # necessary for uncompressed indexes.
         for pkgname in self.keys():
-            yield self[pkgname]
+            pkg = Package(self, self._cache[pkgname])
+            yield self._weakref.setdefault(pkgname, pkg)
 
     def __is_real_pkg(self, rawpkg):
+        # type: (apt_pkg.Package) -> bool
         """Check if the apt_pkg.Package provided is a real package."""
         return rawpkg.has_versions
 
@@ -315,7 +346,7 @@ class Cache(object):
     def __contains__(self, key):
         # type: (object) -> bool
         try:
-            return self.__is_real_pkg(self._cache[key])
+            return self.__is_real_pkg(self._cache[str(key)])
         except KeyError:
             return False
 
@@ -324,7 +355,7 @@ class Cache(object):
         return len(self.keys())
 
     def keys(self):
-        # FIXME: type: () -> List[str] - does not work
+        # type: () -> List[str]
         if self._sorted_set is None:
             self._sorted_set = sorted(p.get_fullname(pretty=True)
                                       for p in self._cache.packages
@@ -423,16 +454,6 @@ class Cache(object):
         # fetched
         return self._run_fetcher(fetcher)
 
-    def _get_archive_lock(self, fetcher):
-        # type: (apt_pkg.Acquire) -> None
-        # get lock
-        archive_dir = apt_pkg.config.find_dir("Dir::Cache::Archives")
-        try:
-            fetcher.get_lock(archive_dir)
-        except apt_pkg.Error as e:
-            raise LockFailedException(("Failed to lock archive directory %s: "
-                                       " %s") % (archive_dir, e))
-
     def fetch_archives(self, progress=None, fetcher=None):
         # type: (AcquireProgress, apt_pkg.Acquire) -> int
         """Fetch the archives for all packages marked for install/upgrade.
@@ -454,10 +475,9 @@ class Cache(object):
         if fetcher is None:
             fetcher = apt_pkg.Acquire(progress)
 
-        self._get_archive_lock(fetcher)
-
-        return self._fetch_archives(fetcher,
-                                    apt_pkg.PackageManager(self._depcache))
+        with self._archive_lock:
+            return self._fetch_archives(fetcher,
+                                        apt_pkg.PackageManager(self._depcache))
 
     def is_virtual_package(self, pkgname):
         # type: (str) -> bool
@@ -504,7 +524,7 @@ class Cache(object):
 
     def update(self, fetch_progress=None, pulse_interval=0,
                raise_on_error=True, sources_list=None):
-        # FIXME: type: (AcquireProgress, int, bool, str) -> int
+        # type: (AcquireProgress, int, bool, str) -> int
         """Run the equivalent of apt-get update.
 
         You probably want to call open() afterwards, in order to utilise the
@@ -517,43 +537,41 @@ class Cache(object):
         sources_list -- Update a alternative sources.list than the default.
         Note that the sources.list.d directory is ignored in this case
         """
-        lockfile = apt_pkg.config.find_dir("Dir::State::Lists") + "lock"
-        lock = apt_pkg.get_lock(lockfile)
-
-        if lock < 0:
-            raise LockFailedException("Failed to lock %s" % lockfile)
-
-        if sources_list:
-            old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
-            old_sources_list_d = apt_pkg.config.find("Dir::Etc::sourceparts")
-            old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
-            apt_pkg.config.set("Dir::Etc::sourcelist",
-                               os.path.abspath(sources_list))
-            apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
-            apt_pkg.config.set("APT::List-Cleanup", "0")
-            slist = apt_pkg.SourceList()
-            slist.read_main_list()
-        else:
-            slist = self._list
-
-        try:
-            if fetch_progress is None:
-                fetch_progress = apt.progress.base.AcquireProgress()
-            try:
-                res = self._cache.update(fetch_progress, slist,
-                                         pulse_interval)
-            except SystemError as e:
-                raise FetchFailedException(e)
-            if not res and raise_on_error:
-                raise FetchFailedException()
-            else:
-                return res
-        finally:
-            os.close(lock)
+        with _WrappedLock(apt_pkg.config.find_dir("Dir::State::Lists")):
             if sources_list:
-                apt_pkg.config.set("Dir::Etc::sourcelist", old_sources_list)
-                apt_pkg.config.set("Dir::Etc::sourceparts", old_sources_list_d)
-                apt_pkg.config.set("APT::List-Cleanup", old_cleanup)
+                old_sources_list = apt_pkg.config.find("Dir::Etc::sourcelist")
+                old_sources_list_d = (
+                    apt_pkg.config.find("Dir::Etc::sourceparts"))
+                old_cleanup = apt_pkg.config.find("APT::List-Cleanup")
+                apt_pkg.config.set("Dir::Etc::sourcelist",
+                                   os.path.abspath(sources_list))
+                apt_pkg.config.set("Dir::Etc::sourceparts", "xxx")
+                apt_pkg.config.set("APT::List-Cleanup", "0")
+                slist = apt_pkg.SourceList()
+                slist.read_main_list()
+            else:
+                slist = self._list
+
+            try:
+                if fetch_progress is None:
+                    fetch_progress = apt.progress.base.AcquireProgress()
+                try:
+                    res = self._cache.update(fetch_progress, slist,
+                                             pulse_interval)
+                except SystemError as e:
+                    raise FetchFailedException(e)
+                if not res and raise_on_error:
+                    raise FetchFailedException()
+                else:
+                    return res
+            finally:
+                if sources_list:
+                    apt_pkg.config.set("Dir::Etc::sourcelist",
+                                       old_sources_list)
+                    apt_pkg.config.set("Dir::Etc::sourceparts",
+                                       old_sources_list_d)
+                    apt_pkg.config.set("APT::List-Cleanup",
+                                       old_cleanup)
 
     def install_archives(self, pm, install_progress):
         # type: (apt_pkg.PackageManager, InstallProgress) -> int
@@ -617,25 +635,25 @@ class Cache(object):
         with apt_pkg.SystemLock():
             pm = apt_pkg.PackageManager(self._depcache)
             fetcher = apt_pkg.Acquire(fetch_progress)
-            self._get_archive_lock(fetcher)
+            with self._archive_lock:
+                while True:
+                    # fetch archives first
+                    res = self._fetch_archives(fetcher, pm)
 
-            while True:
-                # fetch archives first
-                res = self._fetch_archives(fetcher, pm)
-
-                # then install
-                res = self.install_archives(pm, install_progress)
-                if res == pm.RESULT_COMPLETED:
-                    break
-                elif res == pm.RESULT_FAILED:
-                    raise SystemError("installArchives() failed")
-                elif res == pm.RESULT_INCOMPLETE:
-                    pass
-                else:
-                    raise SystemError("internal-error: unknown result code "
-                                      "from InstallArchives: %s" % res)
-                # reload the fetcher for media swaping
-                fetcher.shutdown()
+                    # then install
+                    res = self.install_archives(pm, install_progress)
+                    if res == pm.RESULT_COMPLETED:
+                        break
+                    elif res == pm.RESULT_FAILED:
+                        raise SystemError("installArchives() failed")
+                    elif res == pm.RESULT_INCOMPLETE:
+                        pass
+                    else:
+                        raise SystemError("internal-error: unknown result "
+                                          "code from InstallArchives: %s" %
+                                          res)
+                    # reload the fetcher for media swaping
+                    fetcher.shutdown()
         return (res == pm.RESULT_COMPLETED)
 
     def clear(self):
@@ -657,6 +675,7 @@ class Cache(object):
         self._run_callbacks("cache_pre_change")
 
     def connect(self, name, callback):
+        # type: (str, Union[Callable[..., None],str]) -> None
         """Connect to a signal.
 
         .. deprecated:: 1.0
@@ -672,6 +691,7 @@ class Cache(object):
         self._callbacks[name].append(callback)
 
     def connect2(self, name, callback, *args, **kwds):
+        # type: (str, Callable[..., Any], object, object) -> None
         """Connect to a signal.
 
         The callback will be passed the cache as an argument, and
@@ -900,7 +920,7 @@ class FilteredCache(object):
             yield self.cache[pkgname]
 
     def keys(self):
-        # FIXME: type: () -> List[str] - does not work
+        # type: () -> KeysView[str]
         return self._helper._filtered.keys()
 
     def has_key(self, key):
@@ -927,19 +947,23 @@ class FilteredCache(object):
         self._helper.filter_cache_post_change(self.cache)
 
     def __getattr__(self, key):
+        # type: (str) -> Any
         """we try to look exactly like a real cache."""
         return getattr(self.cache, key)
 
 
 def cache_pre_changed(cache):
+    # type: (Cache) -> None
     print("cache pre changed")
 
 
 def cache_post_changed(cache):
+    # type: (Cache) -> None
     print("cache post changed")
 
 
 def _test():
+    # type: () -> None
     """Internal test code."""
     print("Cache self test")
     apt_pkg.init()
@@ -978,7 +1002,7 @@ def _test():
     filtered.set_filter(MarkedChangesFilter())
     print(len(filtered))
     for pkgname in filtered.keys():
-        assert pkgname == filtered[pkg].name
+        assert pkgname == filtered[pkgname].name
 
     print(len(filtered))
 
