@@ -6,14 +6,13 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 
 import errno
-import ipaddress
 import logging
 import os
 import re
 from functools import partial
 
-from cloudinit import util
 from cloudinit.net.network_state import mask_to_net_prefix
+from cloudinit import util
 from cloudinit.url_helper import UrlError, readurl
 
 LOG = logging.getLogger(__name__)
@@ -308,9 +307,6 @@ def device_devid(devname):
 
 
 def get_devicelist():
-    if util.is_FreeBSD():
-        return list(get_interfaces_by_mac().values())
-
     try:
         devs = os.listdir(get_sys_class_path())
     except OSError as e:
@@ -333,45 +329,6 @@ def is_disabled_cfg(cfg):
 
 def find_fallback_nic(blacklist_drivers=None):
     """Return the name of the 'fallback' network device."""
-    if util.is_FreeBSD():
-        return find_fallback_nic_on_freebsd(blacklist_drivers)
-    elif util.is_NetBSD() or util.is_OpenBSD():
-        return find_fallback_nic_on_netbsd_or_openbsd(blacklist_drivers)
-    else:
-        return find_fallback_nic_on_linux(blacklist_drivers)
-
-
-def find_fallback_nic_on_netbsd_or_openbsd(blacklist_drivers=None):
-    values = list(sorted(
-        get_interfaces_by_mac().values(),
-        key=natural_sort_key))
-    if values:
-        return values[0]
-
-
-def find_fallback_nic_on_freebsd(blacklist_drivers=None):
-    """Return the name of the 'fallback' network device on FreeBSD.
-
-    @param blacklist_drivers: currently ignored
-    @return default interface, or None
-
-
-    we'll use the first interface from ``ifconfig -l -u ether``
-    """
-    stdout, _stderr = util.subp(['ifconfig', '-l', '-u', 'ether'])
-    values = stdout.split()
-    if values:
-        return values[0]
-    # On FreeBSD <= 10, 'ifconfig -l' ignores the interfaces with DOWN
-    # status
-    values = list(get_interfaces_by_mac().values())
-    values.sort()
-    if values:
-        return values[0]
-
-
-def find_fallback_nic_on_linux(blacklist_drivers=None):
-    """Return the name of the 'fallback' network device on Linux."""
     if not blacklist_drivers:
         blacklist_drivers = []
 
@@ -808,74 +765,6 @@ def get_ib_interface_hwaddr(ifname, ethernet_format):
 
 
 def get_interfaces_by_mac():
-    if util.is_FreeBSD():
-        return get_interfaces_by_mac_on_freebsd()
-    elif util.is_NetBSD():
-        return get_interfaces_by_mac_on_netbsd()
-    elif util.is_OpenBSD():
-        return get_interfaces_by_mac_on_openbsd()
-    else:
-        return get_interfaces_by_mac_on_linux()
-
-
-def get_interfaces_by_mac_on_freebsd():
-    (out, _) = util.subp(['ifconfig', '-a', 'ether'])
-
-    # flatten each interface block in a single line
-    def flatten(out):
-        curr_block = ''
-        for line in out.split('\n'):
-            if line.startswith('\t'):
-                curr_block += line
-            else:
-                if curr_block:
-                    yield curr_block
-                curr_block = line
-        yield curr_block
-
-    # looks for interface and mac in a list of flatten block
-    def find_mac(flat_list):
-        for block in flat_list:
-            m = re.search(
-                r"^(?P<ifname>\S*): .*ether\s(?P<mac>[\da-f:]{17}).*",
-                block)
-            if m:
-                yield (m.group('mac'), m.group('ifname'))
-    results = {mac: ifname for mac, ifname in find_mac(flatten(out))}
-    return results
-
-
-def get_interfaces_by_mac_on_netbsd():
-    ret = {}
-    re_field_match = (
-            r"(?P<ifname>\w+).*address:\s"
-            r"(?P<mac>([\da-f]{2}[:-]){5}([\da-f]{2})).*")
-    (out, _) = util.subp(['ifconfig', '-a'])
-    if_lines = re.sub(r'\n\s+', ' ', out).splitlines()
-    for line in if_lines:
-        m = re.match(re_field_match, line)
-        if m:
-            fields = m.groupdict()
-            ret[fields['mac']] = fields['ifname']
-    return ret
-
-
-def get_interfaces_by_mac_on_openbsd():
-    ret = {}
-    re_field_match = (
-        r"(?P<ifname>\w+).*lladdr\s"
-        r"(?P<mac>([\da-f]{2}[:-]){5}([\da-f]{2})).*")
-    (out, _) = util.subp(['ifconfig', '-a'])
-    if_lines = re.sub(r'\n\s+', ' ', out).splitlines()
-    for line in if_lines:
-        m = re.match(re_field_match, line)
-        if m:
-            fields = m.groupdict()
-            ret[fields['mac']] = fields['ifname']
-    return ret
-
-
-def get_interfaces_by_mac_on_linux():
     """Build a dictionary of tuples {mac: name}.
 
     Bridges and any devices that have a 'stolen' mac are excluded."""
@@ -962,38 +851,6 @@ def has_url_connectivity(url):
     return True
 
 
-def is_ip_address(s: str) -> bool:
-    """Returns a bool indicating if ``s`` is an IP address.
-
-    :param s:
-        The string to test.
-
-    :return:
-        A bool indicating if the string contains an IP address or not.
-    """
-    try:
-        ipaddress.ip_address(s)
-    except ValueError:
-        return False
-    return True
-
-
-def is_ipv4_address(s: str) -> bool:
-    """Returns a bool indicating if ``s`` is an IPv4 address.
-
-    :param s:
-        The string to test.
-
-    :return:
-        A bool indicating if the string contains an IPv4 address or not.
-    """
-    try:
-        ipaddress.IPv4Address(s)
-    except ValueError:
-        return False
-    return True
-
-
 class EphemeralIPv4Network(object):
     """Context manager which sets up temporary static network configuration.
 
