@@ -66,6 +66,11 @@ def add_info(report, ui):
         # might happen on local installs
         pass
 
+    # We want to know if people have modified apport's crashdb.conf in case
+    # crashes are reported to Launchpad when they shouldn't be e.g. for a
+    # non-development release.
+    apport.hookutils.attach_conffiles(report, 'apport', ui=ui)
+
     if report.get('ProblemType') == 'Package':
         # every error report regarding a package should have package manager
         # version information
@@ -82,6 +87,8 @@ and installing updates. Please free some space.' % (free_mb)
 
     match_error_messages(report)
 
+    # these attachments will not exist if ProblemType is Bug as the package
+    # hook runs after the general hook
     for attachment in ['DpkgTerminalLog', 'VarLogDistupgradeApttermlog']:
         if attachment in report:
             log_file = get_attachment_contents(report, attachment)
@@ -489,10 +496,10 @@ def trim_dpkg_log(report):
     lines = []
     dpkg_log = report['DpkgTerminalLog']
     if isinstance(dpkg_log, bytes):
-        trim_re = re.compile(b'^\(.* ... \d+ .*\)$')
+        trim_re = re.compile(b'^\\(.* ... \\d+ .*\\)$')
         start_re = re.compile(b'^Log started:')
     else:
-        trim_re = re.compile('^\(.* ... \d+ .*\)$')
+        trim_re = re.compile('^\\(.* ... \\d+ .*\\)$')
         start_re = re.compile('^Log started:')
     for line in dpkg_log.splitlines():
         if start_re.match(line) or trim_re.match(line):
